#!/usr/bin/perl

use XML::Parser;
use File::Basename;
use utf8;

binmode(STDOUT, ":utf8");
### ============================================================================
### Initialisations
### ============================================================================
my $P = new XML::Parser(
         ProtocolEncoding => 'UTF-8',
         Handlers => {Start => \&Entree, End => \&Sortie, Char => \&Caractere});

my @Node = ();
my @Text = ();
my %Sect = ();
my $Tree = {};

### ============================================================================
### Handlers
### ============================================================================
sub Entree {
  my ($x, $e, %a) = @_;
  my $n = $x->depth; $Node[$n] = $e;
}

sub Sortie {
  my ($x, $e) = @_;
  my $n = $x->depth;
  if ($e eq "name") {
    if ($Node[$n-1] eq "Folder") {
      $Tree->{Dossiers} = [] unless exists $Tree->{Dossiers};
      my $r = {}; $r->{Nom} = $Text[$n]; $r->{Pere} = $Tree;
      push @{$Tree->{Dossiers}}, $r;
      $Tree = $r;
    } elsif ($Node[$n-1] eq "Placemark") {
      $Tree->{Lieux} = [] unless exists $Tree->{Lieux};
      my $r = {}; $r->{Nom} = $Text[$n]; $r->{Pere} = $Tree;
      push @{$Tree->{Lieux}}, $r;
      $Tree = $r;
    }
  }
  if ($e eq "Point") {
    if ($Node[$n-1] eq "Placemark") {
      ($Tree->{Point} = $Text[$n]) =~ s/^\s*|\s*//g;
    }
  }
  if ($e eq "LineString") {
    $Tree->{Lignes} = [] unless exists $Tree->{Lignes};
    $Text[$n] =~ s/^\s*|\s*$//g;
    push @{$Tree->{Lignes}}, $Text[$n];
  }
  ($e eq "Folder" or $e eq "Placemark") and $Tree = $Tree->{Pere};
  # --- Libération du niveau ---------------------------------------------------
  $Text[$n-1] = $Text[$n] if $e eq "coordinates";
  $Text[$n] = "";
  $Node[$n] = "";
}

sub Caractere {
  my ($x, $s) = @_;
  my $n = $x->depth;
  $Text[$n - 1] .= $s if $n;
}

### ============================================================================
$P->parsefile("doc.kml");
### ============================================================================


### ============================================================================
### Analyse de l'arbre
### ============================================================================
my %Departements = ();

# Référence sur la structure des données.
my $data = $Tree->{Dossiers}->[0]->{Dossiers};
  # foreach (@$data) { print STDOUT "---> $_->{Nom} <--\n"; }


### Acquisition des noms des départements --------------------------------------
### ----------------------------------------------------------------------------
# Dans le même temps : CamelNotation des départements et enregistrement dans un
# fichier qui sera lu plus tard.
open DTA, ">departements.txt";
binmode(DTA, ":utf8");
my $noms = $data->[0]->{Lieux};
foreach (@$noms) {
  my $nom   = $_->{Nom};
  my $point = $_->{Point};
  my @id = split(/\s+\-\s+/, $nom);
  my $id = $id[0]; $nom = $id[1];
  open TXT, ">Noms/$id.txt";
  binmode(TXT, ":utf8");
  print TXT "$nom\n$point\n";
  close TXT;
  $Departements{$id} = {
    Nom => $nom,
    Coords => $point
  };
  $nom =~ s/ /\-/g;
  $nom =~ s/é/e/g;
  $nom =~ s/è/e/g;
  $nom =~ s/ê/e/g;
  $nom =~ s/ô/o/g;
  $nom =~ s/\-e/E/g;
  $nom =~ s/\-d/D/g;
  $nom =~ s/\-//g;
  $nom =~ s/\'//g;
  print DTA "$id $nom\n";
}
close DTA;

### Acquisition des préfectures ------------------------------------------------
### ----------------------------------------------------------------------------
my $prefs = $data->[1]->{Lieux};
foreach (@$prefs) {
  my $nom   = $_->{Nom};
  my $point = $_->{Point};
  my @id = split(/\s+/, $nom);
  my $id = pop @id;
  my $pref = join(" ",@id);
  open  TXT, ">Prefectures/$id.txt";
  binmode(TXT, ":utf8");
  print TXT "$pref\n$point\n";
  close TXT;
  $Departements{$id}->{Prefecture} = $pref;
  $Departements{$id}->{CoordsPrefecture} = $point;
}

### Acquisition des limites administratives ------------------------------------
### ----------------------------------------------------------------------------
my $limits = $data->[2]->{Dossiers};
foreach (@$limits) {
  my $nom = $_->{Nom};
  my $id  = (split / /,$nom)[0];
  open TXT, ">Limites/$id.txt";
  my $l = $_->{Lieux}->[0]->{Lignes};
  $Departements{$id}->{Limites} = [];
  foreach (@$l) {
    print TXT "$_\n";
    push @{$Departements{$id}->{Limites}}, $_;
  }
  close TXT;
}


### ============================================================================
### Production des fichiers
### ============================================================================
require "lambert93fr.pl";       # Projection conique Lambert93
require "fichiers-mp.pl";        # Fichiers metapost
require "fichiers-dta.pl";       # Fichiers dta

my $index = 1;
open TXT, "departements.txt";
while (<TXT>) {
  chomp;
  my ($id, $nom) = split / /;
  &fichiermp(\%Departements,$id,$nom,$index);
  &fichierdta(\%Departements,$id,$nom);
  $index++;
}

### ==== FIN --------------------------------------------------------------------